@@ -328,7 +328,7 @@ def adaptive_format(num: numbers.Number, prev_value: Union[numbers.Number, Itera
                 return '{0:{1}}'.format(num, '.3f')
 
         elif value_type != 'loss':
-            if 10000< num or  num <= -10000 :
+            if 10000<= num or  num <= -10000 :
                 return '{0:{1}}'.format(num, '.3e')
             else:
                 return '{0:{1}}'.format(num, '.3f')
