@@ -640,10 +640,7 @@ class ModelBase(object):
         model = self.training_context['current_model']
         if is_tensor(model):
             data['output'] = self._model
-            if is_training and self.use_output_as_loss:
-                this_loss = data['output'].sum()
-                self.training_context['losses'].collect(model.signature.outputs.key_list[0], self.training_context['steps'], this_loss)
-                self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+
         else:
             if not is_training:
                 model.eval()
@@ -666,10 +663,10 @@ class ModelBase(object):
                     data[self._signature.outputs.key_list[0]] = output
                 else:
                     data[self._signature.outputs.key_list[0]] = output
-                if is_training and self.use_output_as_loss:
-                    this_loss = output.sum()
-                    self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
-                    self.training_context['losses'].collect(signature.outputs.key_list[0], self.training_context['steps'], to_scalar(this_loss.copy()))
+        if is_training and self.use_output_as_loss and is_tensor(data['output']):
+            this_loss = data['output'].sum()
+            self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+            self.training_context['losses'].collect('output', self.training_context['steps'], to_scalar(this_loss.copy()))
 
     def do_on_loss_calculation_start(self):
         for callback in self.callbacks:
