@@ -394,7 +394,7 @@ class EarlyStoppingCriterionCallback(StoppingCriterionCallback):
         self.monitor_op=None
         self.training_items=None
         self.stopped_epoch=stopped_epoch
-        self._reset()
+
 
     def _reset(self):
         """Resets wait counter and cooldown counter.
@@ -415,8 +415,11 @@ class EarlyStoppingCriterionCallback(StoppingCriterionCallback):
 
 
     def on_training_start(self, training_context):
-        self.wait = 0
-        self.best = 0
+        self._reset()
+        if 'num_epochs' in training_context:
+            stopped_epoch=training_context['num_epochs']
+        elif 'total_epoch'  in training_context:
+            stopped_epoch=training_context['total_epoch']
 
 
     def on_overall_epoch_end(self, training_context):
