@@ -2,7 +2,7 @@ import builtins
 import os
 import copy
 import math
-import numbers
+
 import random
 import inspect
 import string
@@ -19,6 +19,7 @@ from trident.backend.common import *
 from trident.backend.tensorspec import TensorSpec, object_type_inference
 from trident.data.utils import download_file_from_google_drive, unpickle
 from trident.backend import numpy_ops
+import numbers
 
 if get_backend() == 'pytorch':
     #from trident.backend.pytorch_backend import get_device
@@ -28,11 +29,12 @@ elif get_backend() == 'tensorflow':
     from trident.backend.tensorflow_ops import *
 from trident.data.transform import TextTransform
 
-__all__ = ['bpmf_phonetic', 'numbers', 'alphabets', 'chinese_characters', 'RandomSwapChar', 'RandomInsertChar', 'ToHalfWidth', 'RandomMask', 'BopomofoConvert', 'ChineseConvert',
+__all__ = ['bpmf_phonetic', 'numbers_string', 'alphabets', 'chinese_characters', 'RandomSwapChar', 'RandomInsertChar', 'ToHalfWidth', 'RandomMask', 'BopomofoConvert', 'ChineseConvert',
            'RandomHomophonicTypo', 'RandomHomomorphicTypo', 'VocabsMapping']
 
+
 bpmf_phonetic = 'ㄅㄆㄇㄈㄉㄊㄋㄌㄍㄎㄏㄐㄑㄒㄓㄔㄕㄖㄗㄘㄙㄚㄛㄝㄞㄟㄠㄡㄢㄣㄤㄥㄦㄧㄨㄩ'
-numbers = '0123456789'
+numbers_string= '0123456789'
 alphabets = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
 # Ideographic number zero+CJK Unified Ideographs
 chinese_characters = '\u3007'.encode('utf-8').decode('utf-8') + ''.join([chr(i) for i in range(19968, 40959)])
