@@ -680,11 +680,13 @@ def get_activation(fn_name,only_layer=False):
     """
 
     Args:
+        only_layer (bool):
         fn_name ():
 
     Returns:
 
     Examples:
+        >>> a=get_activation('relu')
         >>> print(get_activation('swish'))
 
 
