@@ -22,7 +22,7 @@ from trident.layers.pytorch_layers import *
 from trident.layers.pytorch_normalizations import get_normalization, SpectralNorm
 from trident.layers.pytorch_pooling import *
 from trident.backend.common import *
-from trident.backend.pytorch_backend import Layer, Sequential, ModuleList
+from trident.backend.pytorch_backend import Layer, Sequential, ModuleList,Parameter
 from trident.backend.pytorch_ops import *
 
 __all__ = ['FullConnect_Block','Conv2d_Block', 'Conv1d_Block', 'DepthwiseConv2d_Block', 'SeparableConv2d_Block', 'TemporalConv1d_Block',
@@ -117,6 +117,29 @@ class FullConnect_Block(Layer):
             x = F.dropout(x, p=self.dropout_rate, training=self.training)
         return x
 
+    def fuse(self):
+        if 'batchnorm' in self.norm.__class__.__name__.lower() and not self.use_spectral:
+            with torch.no_grad():
+                bn_rv = self.norm.running_var.data.copy()
+                bn_rm = self.norm.running_mean.data.copy()
+                bn_eps = self.norm.eps.data.copy()
+                bn_w = self.norm.weight.data.copy() if self.norm.affine else ones_like(bn_rm)
+                bn_b = self.norm.bias.data.copy() if self.norm.affine else zeros_like(bn_rm)
+                linear_w = self.fc.weight.data.copy()
+                linear_b = self.fc.bias.data.copy() if self.fc.use_bias else zeros_like(bn_rm)
+                bn_scale = bn_w * torch.rsqrt(bn_rv + bn_eps)
+
+                fused_w = linear_w * bn_scale.unsqueeze(-1)
+                fused_b = (linear_b - bn_rm) * bn_scale + bn_b
+                self.fc.weight.data.copy_(fused_w)
+                self.fc.bias = Parameter(fused_b)
+                # if self.fc.use_bias:
+                #
+                #     self.fc.bias.data.copy_(fused_b)
+                # else:
+
+                self.norm = None
+
     def extra_repr(self):
         s = 'kernel_size={kernel_size}, {num_filters}, strides={strides}'
         if 'activation' in self.__dict__ and self.__dict__['activation'] is not None:
@@ -322,6 +345,31 @@ class Conv2d_Block(Layer):
             x = F.dropout(x, p=self.dropout_rate, training=self.training)
         return x
 
+    def fuse(self):
+
+        if 'batchnorm' in self.norm.__class__.__name__.lower() and not self.use_spectral:
+            with torch.no_grad():
+                bn_rv=self.norm.running_var.data.copy()
+                bn_rm=self.norm.running_mean.data.copy()
+                bn_eps=self.norm.eps.data.copy()
+                conv_w=self.conv.weight.data.copy()
+                conv_b = self.conv.bias.data.copy() if self.conv.use_bias else zeros_like(bn_rm)
+                bn_w=self.norm.weight.data.copy() if self.norm.affine else ones_like(bn_rm)
+                bn_b = self.norm.bias.data.copy() if self.norm.affine else zeros_like(bn_rm)
+                bn_var_rsqrt = torch.rsqrt(bn_rv + bn_eps)
+
+                conv_w = conv_w * (bn_w * bn_var_rsqrt).reshape([-1] + [1] * (len(conv_w.shape) - 1))
+                conv_b = (conv_b - bn_rm) * bn_var_rsqrt * bn_w + bn_b
+                self.conv.weight.data.copy_(conv_w)
+                #self.conv.bias.data.copy_(conv_b) if self.conv.use_bias else \
+                self.conv.bias=Parameter(conv_b)
+                self.norm=None
+
+
+
+
+
+
     def extra_repr(self):
         s = 'kernel_size={kernel_size}, {num_filters}, strides={strides}'
         if 'activation' in self.__dict__ and self.__dict__['activation'] is not None:
