@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import builtins
 import inspect
 import math
 import os
@@ -108,7 +108,7 @@ class _DeeplabV3_plus(Layer):
             if low_level_idx<0 and moduals[i].output_shape[-1]==backbond.input_shape[-1]//8:
                 low_level_idx=i
 
-            if high_level_idx<0 and moduals[i].output_shape[-1]==backbond.input_shape[-1]//32:
+            if high_level_idx<0 and moduals[i].output_shape[-1]==builtins.round(backbond.input_shape[-1]/32,0):
                 high_level_idx=i
                 break
         self.num_filters=num_filters
