@@ -245,6 +245,7 @@ class RFBnet(Layer):
         self.detection_threshold = detection_threshold
         self.nms_threshold =nms_threshold
         self.variance = (center_variance, size_variance)
+        self.softmax = SoftMax(axis=-1)
 
         self.num_classes = num_classes
         self.num_regressors = num_regressors
@@ -308,6 +309,10 @@ class RFBnet(Layer):
         confidence = confidence.permute(0, 2, 3, 1).contiguous()
         confidence = confidence.view(confidence.size(0), -1, self.num_classes)
 
+        if 'softmax' not in self.__dict__:
+            self.softmax=SoftMax(-1)
+        confidence = self.softmax(confidence)
+
         location = self.regression_headers[i](x)
         location = location.permute(0, 2, 3, 1).contiguous()
 
@@ -347,37 +352,41 @@ class RFBnet(Layer):
         area1 = self.area_of(boxes1[..., :2], boxes1[..., 2:])
         return overlap_area / (area0 + area1 - overlap_area + eps)
 
-    def hard_nms(self, box_scores, iou_threshold, top_k=-1, candidate_size=200):
+    def hard_nms(self, box_scores, nms_threshold, top_k=-1, candidate_size=200):
         """
 
         Args:
             box_scores (N, 5): boxes in corner-form and probabilities.
-            iou_threshold: intersection over union threshold.
+            nms_threshold: intersection over union threshold.
             top_k: keep top_k results. If k <= 0, keep all the results.
             candidate_size: only consider the candidates with the highest scores.
         Returns:
              picked: a list of indexes of the kept boxes
         """
+        if box_scores is None or len(box_scores) == 0:
+            return None, None
         scores = box_scores[:, -1]
-        boxes = box_scores[:, :-1]
+        boxes = box_scores[:, :4]
         picked = []
-        _, indexes = scores.sort(descending=True)
-        indexes = indexes[:candidate_size]
+        # _, indexes = scores.sort(descending=True)
+        indexes = np.argsort(-scores)
+        # indexes = indexes[:candidate_size]
+        indexes = indexes[-candidate_size:]
         while len(indexes) > 0:
-            current = indexes[0]
-            picked.append(current.item())
+            # current = indexes[0]
+            current = indexes[-1]
+            picked.append(current)
             if 0 < top_k == len(picked) or len(indexes) == 1:
                 break
             current_box = boxes[current, :]
-            indexes = indexes[1:]
+            # indexes = indexes[1:]
+            indexes = indexes[:-1]
             rest_boxes = boxes[indexes, :]
-            iou = self.iou_of(
-                rest_boxes,
-                current_box.unsqueeze(0),
-            )
-            indexes = indexes[iou <= iou_threshold]
+            iou, inter, union = self.iou_of(rest_boxes, expand_dims(current_box, axis=0), )
+            iot = inter / self.area_of(current_box[..., :2], current_box[..., 2:])
 
-        return box_scores[picked, :]
+            indexes = indexes[((iou <= nms_threshold) * (iot <= nms_threshold)).astype(np.bool)]
+        return box_scores[picked, :], picked
 
     def predict(self, width, height, confidences, boxes, detection_threshold=None, iou_threshold=0.3, top_k=-1):
         boxes = boxes
@@ -433,34 +442,38 @@ class RFBnet(Layer):
         locations = []
 
         x = self.backbond1(x)
-        confidence, location = self.compute_header(0, x)
-        confidences.append(confidence)
-        locations.append(location)
+        confidence0, location0 = self.compute_header(0, x)
+        confidences.append(confidence0)
+        locations.append(location0)
 
         x = self.backbond2(x)
-        confidence, location = self.compute_header(1, x)
-        confidences.append(confidence)
-        locations.append(location)
+        confidence1, location1 = self.compute_header(1, x)
+        confidences.append(confidence1)
+        locations.append(location1)
 
         x = self.backbond3(x)
-        confidence, location = self.compute_header(2, x)
-        confidences.append(confidence)
-        locations.append(location)
+        confidence2, location2 = self.compute_header(2, x)
+        confidences.append(confidence2)
+        locations.append(location2)
 
         x = self.extra(x)
-        confidence, location = self.compute_header(3, x)
-        confidences.append(confidence)
-        locations.append(location)
+        confidence3, location3 = self.compute_header(3, x)
+        confidences.append(confidence3)
+        locations.append(location3)
 
         confidences = torch.cat(confidences, 1)
         locations = torch.cat(locations, 1)
 
-        if self.training:
-            return confidences, locations
-        else:
-            confidences = softmax(confidences, -1)
 
-            locations = decode(locations, self.priors, self.variance)
+        if self.training:
+            result = OrderedDict()
+            result['confidences'] = confidences
+            result['locations'] = locations.view(locations.size(0), -1, self.num_regressors)
+            return result
+
+        else:
+
+            # locations = decode(locations, self.priors, self.variance)
             return confidences, locations
 
 
@@ -473,6 +486,8 @@ def RfbNet(include_top=True,
         input_shape = tuple(input_shape)
     else:
         input_shape = (3, 480, 640)
+    if num_classes != 2 or  num_regressors != 4:
+        pretrained=False
     rfbnet = SsdDetectionModel(input_shape=input_shape, output=RFBnet(base_filters=base_filters, num_classes=num_classes, num_regressors=num_regressors))
     rfbnet.detection_threshold=0.7
     rfbnet.nms_threshold=0.7
@@ -482,16 +497,20 @@ def RfbNet(include_top=True,
         Resize((480, 640), True),
         Normalize(127.5, 127.5)
     ]
+
+
+
     if pretrained == True:
         download_model_from_google_drive('1T_0VYOHaxoyuG1fAxY-6g0C7pfXiujns', dirname, 'version-RFB-640.pth')
         recovery_model =fix_layer( load(os.path.join(dirname, 'version-RFB-640.pth')))
+        recovery_model.softmax = SoftMax(axis=-1)
         priors=recovery_model.priors.clone()
         recovery_model.__delattr__("priors")
         recovery_model.register_buffer("priors",priors)
         recovery_model.name = 'rfb640'
         recovery_model.eval()
         recovery_model.to(_device)
-        rfbnet.model = recovery_model
+        rfbnet.model=recovery_model
     return rfbnet
 
 
