@@ -537,7 +537,7 @@ class Ssd(Layer):
         """
         super(Ssd, self).__init__(name=name)
         self.base_filters = base_filters
-
+        self.softmax = SoftMax(axis=-1)
         idxes = []
         if isinstance(backbond, Sequential):
             for i in range(len(backbond._modules)):
@@ -658,6 +658,7 @@ class Ssd(Layer):
         confidence = self.classification_headers[i](x)
         confidence = confidence.permute(0, 2, 3, 1).contiguous()
         confidence = confidence.view(confidence.size(0), -1, self.num_classes)
+        confidence = self.softmax(confidence)
 
         location = self.regression_headers[i](x)
         location = location.permute(0, 2, 3, 1).contiguous()
@@ -695,11 +696,14 @@ class Ssd(Layer):
         locations = torch.cat(locations, 1)
 
         if self.training:
-            return confidences, locations
-        else:
-            confidences = softmax(confidences, -1)
+            result = OrderedDict()
+            result['confidences'] = confidences
+            result['locations'] = locations.view(locations.size(0), -1, self.num_regressors)
+            return result
 
-            locations = decode(locations, self.priors, self.variance)
+        else:
+
+            # locations = decode(locations, self.priors, self.variance)
             return confidences, locations
 
 
@@ -763,7 +767,7 @@ class SsdDetectionModel(ImageDetectionModel):
         boxes = box_scores[:, :4]
         picked = []
         # _, indexes = scores.sort(descending=True)
-        indexes = argsort(scores)
+        indexes = np.argsort(-scores)
         # indexes = indexes[:candidate_size]
         indexes = indexes[-candidate_size:]
         while len(indexes) > 0:
@@ -776,9 +780,10 @@ class SsdDetectionModel(ImageDetectionModel):
             # indexes = indexes[1:]
             indexes = indexes[:-1]
             rest_boxes = boxes[indexes, :]
-            iou = self.iou_of(rest_boxes, expand_dims(current_box, axis=0), )
-            indexes = indexes[iou <= nms_threshold]
+            iou, inter, union = self.iou_of(rest_boxes, expand_dims(current_box, axis=0), )
+            iot = inter / self.area_of(current_box[..., :2], current_box[..., 2:])
 
+            indexes = indexes[((iou <= nms_threshold) * (iot <= nms_threshold)).astype(np.bool)]
         return box_scores[picked, :], picked
 
     def nms(self, boxes, nms_threshold=0.3):
