@@ -206,7 +206,7 @@ class _ClassificationLoss(Loss):
                 output=output/reduce_max(output)
 
             sum_output=reduce_sum(output,axis=self.axis,keepdims=True)
-            if reduce_max(abs(sum_output-ones_like(sum_output)))<1e-3:
+            if reduce_max(abs(sum_output-1))<1e-3:
                 self.from_logits = True
             else:
                 output=output/clip(sum_output,1e-7,1-1e-7)
@@ -866,7 +866,10 @@ class FocalLoss(_ClassificationLoss):
         normalized (bool): Compute normalized focal loss (https://arxiv.org/pdf/1909.07829.pdf).
         threshold (float, optional): Compute reduced focal loss (https://arxiv.org/abs/1903.01347).
 
+     Examples:
 
+        >>> FocalLoss(reduction='mean',axis=-1)(to_tensor([[0.1, 0.7 , 0.2],[0.3 , 0.6 , 0.1],[0.9 , 0.05 , 0.05],[0.3 , 0.4 , 0.3]]).float(),to_tensor([1,0,1,2]).long()).cpu()
+        tensor(1.1305)
     """
 
     def __init__(self, alpha=0.5, gamma=2, normalized=False, threshold=None, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None,
@@ -892,30 +895,34 @@ class FocalLoss(_ClassificationLoss):
         Returns:
 
             """
-        if self.is_logsoftmax:
-            output = clip(exp(output), 1e-8, 1 - 1e-8)
-            self.from_logits = True
-        if not self.need_target_onehot:
-            if self.is_target_onehot:
-                target = argmax(target, self.axis)
-            probs = exp(output)
-            return F.nll_loss(pow(1 - probs, self.gamma) * output, target, self.sample_weight, ignore_index=self.ignore_index, reduction='none')
-        else:
-            sample_weight = self.sample_weight
-            if ndim(output) == 2:
-                if self.is_logsoftmax == True:
-                    sample_weight = self.sample_weight * self.ignore_index_weight
-            else:
-                reshape_shape = [1] * ndim(output)
-                reshape_shape[self.axis] = self.num_classes
-                sample_weight = self.sample_weight.view(*reshape_shape)
-                if self.is_logsoftmax == True:
-                    sample_weight = self.sample_weight.view(*reshape_shape) * self.ignore_index_weight.view(*reshape_shape)
+        alpha = torch.zeros(self.num_classes).to(output.device)
+        alpha[0] += self.alpha
+        alpha[1:] += (1 - self.alpha)
+        if ndim(output) > 2 and self.axis==1:
+            output = output.view(output.size(0), output.size(1), -1)  # N,C,H,W => N,C,H*W
+            output = output.transpose(1, 2)  # N,C,H*W => N,H*W,C
+            output = output.contiguous().view(-1, output.size(2))  # N,H*W,C => N*H*W,C
+
+
+        if self.is_target_onehot and target.dtype != dtype.long:
+            target = argmax(target, self.axis)
+            self.is_target_onehot=False
+        target = target.view(-1, 1)
+
+        if not self.is_logsoftmax:
+            output=log_softmax(output)
+            self.is_logsoftmax=True
+
+        logpt=output
+        logpt = logpt.gather(1, target)
+        pt = logpt.data.exp()
+        #
+        loss=-1 *  ((1 - pt) ** self.gamma) * logpt
+        alpha=alpha.gather(0, target.view(-1))
+        loss=loss*(alpha.view(-1,1))
+        return loss
 
 
-            probs = exp(output) * sample_weight
-            loss = -self.alpha * target * pow((1 - probs), self.gamma) * output
-            return loss
 
         # - \alpha(1 - softmax(x)[class ]) ^ gamma \log(softmax(x)[class])
 
@@ -1253,19 +1260,14 @@ class AdaptiveWingLoss(_PairwiseLoss):
         Returns:
 
         """
-        y = target
-        y_hat = output
-        delta_y = (y - y_hat).abs()
-        delta_y1 = delta_y[delta_y < self.theta]
-        delta_y2 = delta_y[delta_y >= self.theta]
-        y1 = y[delta_y < self.theta]
-        y2 = y[delta_y >= self.theta]
-        loss1 = self.omega * torch.log(1 + torch.pow(delta_y1 / self.omega, self.alpha - y1))
-        A = self.omega * (1 / (1 + torch.pow(self.theta / self.epsilon, self.alpha - y2))) * (self.alpha - y2) * (
-            torch.pow(self.theta / self.epsilon, self.alpha - y2 - 1)) * (1 / self.epsilon)
-        C = self.theta * A - self.omega * torch.log(1 + torch.pow(self.theta / self.epsilon, self.alpha - y2))
-        loss2 = A * delta_y2 - C
-        return (loss1.sum() + loss2.sum()) / (len(loss1) + len(loss2))
+        lossMat = torch.zeros_like(output,dtype=output.dtype).to(output.device)
+        A = self.omega * (1 / (1 + (self.theta / self.epsilon) ** (self.alpha - target))) * (self.alpha - target) * ((self.theta / self.epsilon) ** (self.alpha - target - 1)) / self.epsilon
+        C = self.theta * A - self.omega * torch.log(1 + (self.theta / self.epsilon) ** (self.alpha - target))
+        case1_ind = torch.abs(target - output) < self.theta
+        case2_ind = torch.abs(target - output) >= self.theta
+        lossMat[case1_ind] = self.omega * torch.log(1 + torch.abs((target[case1_ind] - output[case1_ind]) / self.epsilon) ** (self.alpha - target[case1_ind]))
+        lossMat[case2_ind] = A[case2_ind] * torch.abs(target[case2_ind] - output[case2_ind]) - C[case2_ind]
+        return lossMat
 
 
 class ExponentialLoss(_PairwiseLoss):
@@ -1934,9 +1936,9 @@ class CenterLoss(_Loss):
             dist.append(value)
         dist = torch.cat(dist)
         if self.reduction == 'mean':
-            return dist.mean() / self.num_classes
+            return dist.mean() #/ self.num_classes
         else:
-            return dist.sum() / self.num_classes
+            return dist.sum() #/ self.num_classes
 
 
 class StyleLoss(nn.Module):
