@@ -64,6 +64,30 @@ class Optimizer(optimizer.Optimizer):
         """
         return self.param_groups['params']
 
+    @parameters.setter
+    def parameters(self,value):
+        """
+
+        Returns: the weights need to train
+
+        """
+        if isinstance(value, torch.Tensor):
+            raise TypeError("params argument given to the optimizer should be "
+                            "an iterable of Tensors or dicts, but got " +
+                            torch.typename(value))
+        if not hasattr(self,'param_groups') or self.param_groups is None or len(self.param_groups)==0:
+            self.param_groups=[]
+
+            param_groups = list(value)
+            if len(param_groups) == 0:
+                raise ValueError("optimizer got an empty parameter list")
+            if not isinstance(param_groups[0], dict):
+                param_groups = [{'params': param_groups}]
+                for param_group in param_groups:
+                    self.add_param_group(param_group)
+        else:
+            self.param_groups[0]['params']=value
+
     @property
     def lr(self):
         """str: The getter method of the 'learning rate' property."""
@@ -273,7 +297,7 @@ class SGD(optim.SGD):
     """
 
     def __init__(self, params, lr=1e-3, momentum=0, dampening=0,
-                 weight_decay=0, nesterov=False):
+                 weight_decay=0, nesterov=False,**kwargs):
         super().__init__(params, lr=lr, momentum=momentum, dampening=dampening, weight_decay=weight_decay, nesterov=nesterov)
 
     def adjust_learning_rate(self, new_lr, verbose=True):
@@ -362,7 +386,7 @@ class LBFGS(lbfgs.LBFGS):
                  tolerance_grad=1e-7,
                  tolerance_change=1e-9,
                  history_size=100,
-                 line_search_fn=None):
+                 line_search_fn=None,**kwargs):
         super().__init__(params, lr=lr, max_iter=max_iter, max_eval=max_eval, tolerance_grad=tolerance_grad, tolerance_change=tolerance_change, history_size=history_size,
                          line_search_fn=line_search_fn)
 
@@ -433,7 +457,7 @@ class Adadelta(adadelta.Adadelta):
     __ https://arxiv.org/abs/1212.5701
     """
 
-    def __init__(self, params, lr=1.0, rho=0.9, eps=1e-7, weight_decay=0):
+    def __init__(self, params, lr=1.0, rho=0.9, eps=1e-7, weight_decay=0,**kwargs):
         super().__init__(params, lr=lr, rho=rho, eps=eps, weight_decay=weight_decay)
 
     def adjust_learning_rate(self, new_lr, verbose=True):
@@ -503,7 +527,7 @@ class Adagrad(adagrad.Adagrad):
          Optimization: http://jmlr.org/papers/v12/duchi11a.html
      """
 
-    def __init__(self, params, lr=1e-2, lr_decay=0, weight_decay=0, initial_accumulator_value=0, eps=1e-7):
+    def __init__(self, params, lr=1e-2, lr_decay=0, weight_decay=0, initial_accumulator_value=0, eps=1e-7,**kwargs):
         super().__init__(params, lr=lr, lr_decay=lr_decay, eps=eps, weight_decay=weight_decay, initial_accumulator_value=initial_accumulator_value)
 
     def adjust_learning_rate(self, new_lr, verbose=True):
@@ -583,7 +607,7 @@ class RMSprop(rmsprop.RMSprop):
 
     """
 
-    def __init__(self, params, lr=1e-2, alpha=0.99, eps=1e-7, weight_decay=0, momentum=0, centered=False, gradient_centralization=None):
+    def __init__(self, params, lr=1e-2, alpha=0.99, eps=1e-7, weight_decay=0, momentum=0, centered=False, gradient_centralization=None,**kwargs):
         super().__init__(params, lr=lr, alpha=alpha, eps=eps, weight_decay=weight_decay, momentum=momentum, centered=centered)
 
     def adjust_learning_rate(self, new_lr, verbose=True):
@@ -674,7 +698,7 @@ class RAdam(Optimizer):
         """
 
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-7, weight_decay=0, N_sma_threshhold=5,
-                 degenerated_to_sgd=True, gradient_centralization=None):
+                 degenerated_to_sgd=True, gradient_centralization=None,**kwargs):
         """Construct a new RAdam optimizer.
         Args:
             params: trainable parameters from model
@@ -798,7 +822,7 @@ class RAdam(Optimizer):
 
 
 class PlainRAdam(Optimizer):
-    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, degenerated_to_sgd=True, gradient_centralization=None):
+    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, degenerated_to_sgd=True, gradient_centralization=None,**kwargs):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
         if not 0.0 <= eps:
@@ -913,7 +937,7 @@ class AdamW(Optimizer):
 
     """
 
-    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-7, weight_decay=0, warmup=0, gradient_centralization=None):
+    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-7, weight_decay=0, warmup=0, gradient_centralization=None,**kwargs):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
         if not 0.0 <= eps:
@@ -1005,7 +1029,7 @@ class AdamW(Optimizer):
 
 
 class Lookahead(Optimizer):
-    def __init__(self, optimizer, params, defaults, k=5, alpha=0.5, gradient_centralization=None):
+    def __init__(self, optimizer, params, defaults, k=5, alpha=0.5, gradient_centralization=None,**kwargs):
         super().__init__(params, defaults)
         self.optimizer = optimizer
         self.k = k
@@ -1078,7 +1102,7 @@ class Ranger(Optimizer):
     """
 
     def __init__(self, params, lr=1e-3, alpha=0.5, k=6, N_sma_threshhold=5, betas=(.95, 0.999), eps=1e-5,
-                 weight_decay=0, gradient_centralization=None):
+                 weight_decay=0, gradient_centralization=None,**kwargs):
         self.gradient_centralization = gradient_centralization
         # parameter checks
         if not 0.0 <= alpha <= 1.0:
@@ -1262,7 +1286,7 @@ class RangerLars(Optimizer):
     https://github.com/lessw2020/Ranger-Deep-Learning-Optimizer/blob/master/ranger/ranger.py
     """
 
-    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), alpha=0.5, k=6, N_sma_threshhold=5, eps=1e-7, weight_decay=0, gradient_centralization=None):
+    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), alpha=0.5, k=6, N_sma_threshhold=5, eps=1e-7, weight_decay=0, gradient_centralization=None,**kwargs):
         # parameter checks
         if not 0.0 <= alpha <= 1.0:
             raise ValueError('Invalid slow update rate: {alpha}')
@@ -1448,7 +1472,7 @@ class LARS(Optimizer):
             exclude_from_weight_decay=None,
             exclude_from_layer_adaptation=None,
             classic_momentum=True,
-            eeta=0.001):
+            eeta=0.001,**kwargs):
         """Constructs a LARSOptimizer.
         Args:
         lr: A `float` for learning rate.
@@ -1628,7 +1652,7 @@ class AdaBelief(Optimizer):
 
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-16,
                  weight_decay=0, amsgrad=False, weight_decouple=True, fixed_decay=False, rectify=True,
-                 degenerated_to_sgd=True):
+                 degenerated_to_sgd=True, gradient_centralization=None,**kwargs):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
         if not 0.0 <= eps:
@@ -1652,6 +1676,7 @@ class AdaBelief(Optimizer):
         self.weight_decouple = weight_decouple
         self.rectify = rectify
         self.fixed_decay = fixed_decay
+        self.gradient_centralization=gradient_centralization
 
     def __setstate__(self, state):
         super(AdaBelief, self).__setstate__(state)
@@ -1671,12 +1696,12 @@ class AdaBelief(Optimizer):
                     if version_higher else torch.zeros_like(p.data)
 
                 # Exponential moving average of squared gradient values
-                state['exp_avg_var'] = torch.zeros_like(p.data, memory_format=torch.preserve_format) \
+                state['exp_avg_sq'] = torch.zeros_like(p.data, memory_format=torch.preserve_format) \
                     if version_higher else torch.zeros_like(p.data)
 
                 if amsgrad:
                     # Maintains max of all exp. moving avg. of sq. grad. values
-                    state['max_exp_avg_var'] = torch.zeros_like(p.data, memory_format=torch.preserve_format) \
+                    state['max_exp_avg_sq'] = torch.zeros_like(p.data, memory_format=torch.preserve_format) \
                         if version_higher else torch.zeros_like(p.data)
 
     @torch.no_grad()
@@ -1721,10 +1746,10 @@ class AdaBelief(Optimizer):
                     # Exponential moving average of gradient values
                     state['exp_avg'] = torch.zeros_like(p.data, memory_format=torch.preserve_format) if version_higher else torch.zeros_like(p.data)
                     # Exponential moving average of squared gradient values
-                    state['exp_avg_var'] = torch.zeros_like(p.data, memory_format=torch.preserve_format)  if version_higher else torch.zeros_like(p.data)
+                    state['exp_avg_sq'] = torch.zeros_like(p.data, memory_format=torch.preserve_format)  if version_higher else torch.zeros_like(p.data)
                     if amsgrad:
                         # Maintains max of all exp. moving avg. of sq. grad. values
-                        state['max_exp_avg_var'] = torch.zeros_like(p.data, memory_format=torch.preserve_format)  if version_higher else torch.zeros_like(p.data)
+                        state['max_exp_avg_sq'] = torch.zeros_like(p.data, memory_format=torch.preserve_format)  if version_higher else torch.zeros_like(p.data)
 
                 # perform weight decay, check if decoupled weight decay
                 if self.weight_decouple:
@@ -1753,12 +1778,12 @@ class AdaBelief(Optimizer):
                 exp_avg_sq.mul_(beta2).addcmul_(grad_residual, grad_residual, value=1 - beta2)
 
                 if amsgrad:
-                    max_exp_avg_var = state['max_exp_avg_sq']
+                    max_exp_avg_sq = state['max_exp_avg_sq']
                     # Maintains the maximum of all 2nd moment running avg. till now
-                    torch.max(max_exp_avg_var, exp_avg_sq.add_(group['eps']), out=max_exp_avg_var)
+                    torch.max(max_exp_avg_sq, exp_avg_sq.add_(group['eps']), out=max_exp_avg_sq)
 
                     # Use the max. for normalizing running avg. of gradient
-                    denom = (max_exp_avg_var.sqrt() / math.sqrt(bias_correction2)).add_(group['eps'])
+                    denom = (max_exp_avg_sq.sqrt() / math.sqrt(bias_correction2)).add_(group['eps'])
                 else:
                     denom = (exp_avg_sq.add_(group['eps']).sqrt() / math.sqrt(bias_correction2)).add_(group['eps'])
 
@@ -1854,7 +1879,7 @@ class RangerAdaBelief(Optimizer):
                  alpha=0.5, k=6, N_sma_threshhold=5,  # Ranger options
                  betas=(.95, 0.999), eps=1e-5, weight_decay=0,  # Adam options
                  # Gradient centralization on or off, applied to conv layers only or conv + fc layers
-                 use_gc=True, gc_conv_only=False, gc_loc=True, adabelief=True, weight_decouple=True, **kwargs):
+                 adabelief=True, weight_decouple=True,gradient_centralization=None, **kwargs):
 
         if not 0.0 <= alpha <= 1.0:
             raise ValueError(f'Invalid slow update rate: {alpha}')
@@ -1887,9 +1912,7 @@ class RangerAdaBelief(Optimizer):
         self.radam_buffer = [[None, None, None] for ind in range(10)]
 
         # gc on or off
-        self.gc_loc = gc_loc
-        self.use_gc = use_gc
-        self.gc_conv_only = gc_conv_only
+        self.gradient_centralization=gradient_centralization
         # level of gradient centralization
         # self.gc_gradient_threshold = 3 if gc_conv_only else 1
 
@@ -2179,7 +2202,7 @@ class Lamb(Optimizer):
         https://arxiv.org/abs/1904.00962
     """
 
-    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-6, weight_decay=0, adam=False, gradient_centralization=None):
+    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-6, weight_decay=0, adam=False, gradient_centralization=None,**kwargs):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
         if not 0.0 <= eps:
