@@ -364,9 +364,9 @@ class TrainingPlan(object):
 
         for trainingitem in self.training_items.value_list:
             existing_data_feed = None
-            if 'data_feed' in trainingitem.training_context:
+            if 'data_feed' in trainingitem.training_context and trainingitem.training_context['data_feed'] is not None:
                 existing_data_feed = trainingitem.training_context['data_feed']
-            if not hasattr(trainingitem._model, '_signature') or trainingitem._model._signature is None:
+            if not is_tensor(trainingitem._model) and not hasattr(trainingitem._model, '_signature') or trainingitem._model._signature is None:
                 trainingitem._model._signature = get_signature(trainingitem._model)
 
             data_feed = OrderedDict()
@@ -415,7 +415,8 @@ class TrainingPlan(object):
                         available_items.remove(inp)
                     else:
                         data_feed[inp] = None
-
+            if 'output' in data_feed and data_feed['output'] is None:
+                data_feed['output']='output'
             if 'x' in data_feed and data_feed['x'] is None:
                 if len(data_symbols) == 1 and data_symbols[0] in available_items:
                     data_feed['x'] = data_symbols[0]
